package fucoin;

import akka.actor.ActorRef;
import akka.actor.ActorSelection;
import akka.actor.ActorSystem;
import akka.japi.Pair;
import akka.util.Timeout;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;
import fucoin.setup.SelectableNetworkInterface;
import fucoin.wallet.WalletImpl;
import scala.concurrent.Await;
import scala.concurrent.duration.FiniteDuration;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.net.*;
import java.util.*;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class MainRemote {

    private static JTextField walletNameField = new JTextField(10);
    private static JTextField pathField = new JTextField(10);
    private static JComboBox<SelectableNetworkInterface> hostnameField = new JComboBox<>();

    public static void main(String[] args) throws InterruptedException {

        String hostname = "127.0.0.1";
        try {
            // Fetch IP address to enable network connection
            hostname = InetAddress.getLocalHost().getHostAddress();
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }

        //Load configuration from current directory or from resources directory of jar
        File file = new File("application.conf");
        Config config = ConfigFactory.parseFile(file);
        if (!file.exists()) {
            System.out.println("Load default application.conf");
            config = ConfigFactory.parseResources("application.conf");
        } else {
            System.out.println("Load local application.conf");
        }

        //Init System Actor System
        ActorSystem system = ActorSystem.create("Remote", ConfigFactory.parseString("akka.remote.netty.tcp.hostname=" + hostname).withFallback(config));

        JPanel dialogPanel = createDialogPanel(hostname);

        Timeout timeout = new Timeout(5, TimeUnit.SECONDS);
        ActorRef preknownNeighbour = null;

        String walletName = null;
        String path = null;

        while (preknownNeighbour == null) {

            int result = JOptionPane.showConfirmDialog(null, dialogPanel, "Connect to wallet network", JOptionPane.OK_CANCEL_OPTION);

            // terminate if user clicked cancel
            if (result == JOptionPane.CANCEL_OPTION) {
                if (system != null) {
                    system.terminate();
                }
                return;
            }

            SelectableNetworkInterface selectedHostname = (SelectableNetworkInterface) hostnameField.getSelectedItem();

            if (!selectedHostname.getHostName().equals(hostname)) {
                if (system != null) {
                    try {
                        Await.result(system.terminate(), new FiniteDuration(5, TimeUnit.SECONDS));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                hostname = selectedHostname.getHostName();
                system = ActorSystem.create("Remote", ConfigFactory.parseString("akka.remote.netty.tcp.hostname=" + hostname).withFallback(config));
            }

            walletName = walletNameField.getText();
            path = pathField.getText();

            // check input
            if (path.equals("") || walletName.equals("")) {
                continue;
            }

            // resolve the given address
            ActorSelection selection = system.actorSelection(path);

            try {
                preknownNeighbour = Await.result(selection.resolveOne(timeout), timeout.duration());
            } catch (Exception e) {
                System.err.println("Something went wrong while resolving: " + e.getMessage());
            }
        }

        // spawn wallet
        system.actorOf(WalletImpl.props(preknownNeighbour, walletName), walletName);

    }

    private static JPanel createDialogPanel(String defaultHostname) {
        JPanel dialogPanel = new JPanel();
        dialogPanel.setLayout(new GridLayout(3, 1));
        JLabel walletLabel = new JLabel("Pick your wallet name: ", SwingConstants.LEFT);
        dialogPanel.add(walletLabel);
        dialogPanel.add(walletNameField);
        dialogPanel.add(new JLabel("Enter a neighbour node address: ", SwingConstants.LEFT));
        dialogPanel.add(pathField);
        dialogPanel.add(new JLabel("Select your reachable IP address: ", SwingConstants.LEFT));

        List<SelectableNetworkInterface> interfaces = fetchNetworkInterfaces();

        for (SelectableNetworkInterface netint : interfaces) {
            hostnameField.addItem(netint);
        }

        hostnameField.setSelectedItem(new SelectableNetworkInterface(defaultHostname, "default"));
        dialogPanel.add(hostnameField);
        return dialogPanel;
    }

    private static List<SelectableNetworkInterface> fetchNetworkInterfaces() {
        List<SelectableNetworkInterface> map = new ArrayList<>();

        try {
            Enumeration<NetworkInterface> nets = NetworkInterface.getNetworkInterfaces();

            for (NetworkInterface networkInterface : Collections.list(nets)) {
                Enumeration<InetAddress> inetAddresses = networkInterface.getInetAddresses();
                // To ease the setup, currently only IPv4 addresses are supported
                Optional<InetAddress> picked = Collections.list(inetAddresses).stream().filter(inetAddress -> !(inetAddress instanceof Inet6Address)).findFirst();
                if (picked.isPresent()) {
                    String hostAddress = picked.get().getHostAddress();
                    map.add(new SelectableNetworkInterface(hostAddress, networkInterface.getDisplayName()));
                }
            }
        } catch (SocketException e) {
            e.printStackTrace();
        }

        return map;
    }
}
